@@ -36,7 +36,7 @@
 #define _SIS_H_
 
 #define SISDRIVERVERSIONYEAR    5
-#define SISDRIVERVERSIONMONTH   7
+#define SISDRIVERVERSIONMONTH   9
 #define SISDRIVERVERSIONDAY     20
 #define SISDRIVERREVISION       1
 
@@ -56,14 +56,6 @@
 #endif
 #endif
 
-#define SIS_NAME                "SIS"
-#define SIS_DRIVER_NAME         "sis"
-#define SIS_MAJOR_VERSION       0
-#define SIS_MINOR_VERSION       7
-#define SIS_PATCHLEVEL          0
-#define SIS_CURRENT_VERSION     ((SIS_MAJOR_VERSION << 16) | \
-                                 (SIS_MINOR_VERSION << 8) | SIS_PATCHLEVEL )
-
 #if 0
 #define TWDEBUG    /* for debugging */
 #endif
@@ -87,6 +79,7 @@
 
 #define SIS_HaveDriverFuncs 0
 
+#undef SISISXORG6899900
 #ifdef XORG_VERSION_CURRENT
 #include "xorgVersion.h"
 #define SISMYSERVERNAME "X.org"
@@ -95,6 +88,9 @@
 	(((major) * 10000000) + ((minor) * 100000) + ((patch) * 1000) + snap)
 #define XF86_VERSION_CURRENT XF86_VERSION_NUMERIC(4,3,99,902,0)
 #endif
+#if XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(6,8,99,900,0)
+#define SISISXORG6899900
+#endif
 #if 0
 #ifdef HaveDriverFuncs
 #define SIS_HAVE_DRIVER_FUNC
@@ -107,6 +103,18 @@
 #define SISMYSERVERNAME "XFree86"
 #endif
 
+#define SIS_NAME                "SIS"
+#define SIS_DRIVER_NAME         "sis"
+#define SIS_MAJOR_VERSION       0
+#ifdef SISISXORG6899900
+#define SIS_MINOR_VERSION       8	/* DRI changes */
+#else
+#define SIS_MINOR_VERSION       7
+#endif
+#define SIS_PATCHLEVEL          0
+#define SIS_CURRENT_VERSION     ((SIS_MAJOR_VERSION << 16) | \
+                                 (SIS_MINOR_VERSION << 8) | SIS_PATCHLEVEL )
+
 #if (XF86_VERSION_CURRENT >= XF86_VERSION_NUMERIC(4,3,99,0,0)) || (defined(XvExtension))
 #include "xf86xv.h"
 #include <X11/extensions/Xv.h>
@@ -201,7 +209,7 @@
 
 #ifdef SISVRAMQ
 #ifdef XORG_VERSION_CURRENT
-#ifdef SIS_HAVE_EXA
+#if defined(SIS_HAVE_EXA) || (defined(XF86EXA) && (XF86EXA != 0))
 #if 1
 #define SIS_USE_EXA		/* Include code for EXA */
 #endif
@@ -350,8 +358,8 @@
 #define CRT1_LCDA		0x00020000
 #define VGA2_CONNECTED		0x00040000
 #define DISPTYPE_CRT1		0x00080000  	/* CRT1 connected and used */
-#define OLDVB_301		0x00100000	/* Definition deprecated (now VBFlags2) */
-#define OLDVB_301B		0x00200000	/* Definition deprecated (now VBFlags2) */
+#define TV_YPBPR625I		0x00100000
+#define TV_YPBPR625P		0x00200000
 #define OLDVB_302B		0x00400000	/* Definition deprecated (now VBFlags2) */
 #define OLDVB_30xBDH		0x00800000      /* Definition deprecated (now VBFlags2) */
 #define OLDVB_LVDS		0x01000000	/* Definition deprecated (now VBFlags2) */
@@ -373,7 +381,9 @@
 #define TV_YPBPR525P		TV_PAL
 #define TV_YPBPR750P		TV_PALM
 #define TV_YPBPR1080I		TV_PALN
-#define TV_YPBPRALL 		(TV_YPBPR525I | TV_YPBPR525P | TV_YPBPR750P | TV_YPBPR1080I)
+#define TV_YPBPRALL 		(TV_YPBPR525I | TV_YPBPR525P | \
+				 TV_YPBPR625I | TV_YPBPR625P | \
+				 TV_YPBPR750P | TV_YPBPR1080I)
 
 #define TV_YPBPR43LB		TV_CHSCART
 #define TV_YPBPR43		TV_CHYPBPR525I
@@ -427,10 +437,33 @@
 #define VB2_30xCLV		(VB2_301C  | VB2_307T   | VB2_302ELV| VB2_307LV)
 #define VB2_SISEMIBRIDGE	(VB2_302LV | VB2_302ELV | VB2_307LV)
 #define VB2_LCD162MHZBRIDGE	(VB2_301C  | VB2_307T)
+/* CRT2/LCD over 1280 (overflow bits in Part4) */
 #define VB2_LCDOVER1280BRIDGE	(VB2_301C  | VB2_307T   | VB2_302LV | VB2_302ELV | VB2_307LV)
+/* CRT2/LCD over 1600? Is this really gonna happen, or will there be LCDA only for large panels? */
 #define VB2_LCDOVER1600BRIDGE	(VB2_307T  | VB2_307LV)
+/* VGA2 up to 202MHz (1600x1200@75) */
 #define VB2_RAMDAC202MHZBRIDGE	(VB2_301C  | VB2_307T)
 
+/* pSiS->VBFlags3 (for future use) */
+#define VB3_CRT1_TV		0x00000001
+#define VB3_CRT1_LCD		0x00000002
+#define VB3_CRT1_VGA		0x00000004
+#define TV1_AVIDEO		0x00000100
+#define TV1_SVIDEO		0x00000200
+#define TV1_SCART		0x00000400
+#define TV1_NTSC		0x00000800
+#define TV1_PAL			0x00001000
+#define TV1_YPBPR		0x00002000
+#define TV1_PALM		0x00004000
+#define TV1_PALN		0x00008000
+#define TV1_NTSCJ		0x00010000
+#define TV1_YPBPR525I		0x00020000
+#define TV1_YPBPR525P		0x00040000
+#define TV1_YPBPR625I		0x00080000
+#define TV1_YPBPR625P		0x00100000
+#define TV1_YPBPR750P		0x00200000
+#define TV1_YPBPR1080I		0x00400000
+
 /* pSiS->VBLCDFlags */
 #define VB_LCD_320x480		0x00000001	/* DSTN/FSTN for 550 */
 #define VB_LCD_640x480		0x00000002
@@ -623,9 +656,26 @@ typedef unsigned char  UChar;
 #define SiS_SD2_SUPPORTXVDEINT 0x00080000   /* Xv deinterlacing supported (n/a, for future use) */
 #define SiS_SD2_ISXGI	       0x00100000   /* Is XGI chip */
 #define SiS_SD2_USEVBFLAGS2    0x00200000   /* Use VBFlags2 for bridge ID */
-/* ... */
+#define SiS_SD2_SUPPLTFLAG     0x00400000   /* Driver supports the following 3 flags */
+#define SiS_SD2_ISLAPTOP       0x00800000   /* This machine is (very probably) a laptop */
+#define SiS_SD2_MACHINETYPE2   0x01000000   /* Machine type 2 (for future use) */
+#define SiS_SD2_MACHINETYPE3   0x02000000   /* Machine type 3 (for future use) */
+#define SiS_SD2_SUPPORT625I    0x04000000   /* Support YPbPr 625i */
+#define SiS_SD2_SUPPORT625P    0x08000000   /* Support YPbPr 625p */
+#define SiS_SD2_VBINVB2ONLY    0x10000000   /* VB_* bits in vbflags no longer used for vb type */
+#define SiS_SD2_NEWGAMMABRICON 0x20000000   /* Support new gamma brightness/contrast */
+#define SiS_SD2_HAVESD34       0x40000000   /* Support SD3 and SD4 flags */
 #define SiS_SD2_NOOVERLAY      0x80000000   /* No video overlay */
 
+#define SiS_SD3_OLDGAMMAINUSE  0x00000001   /* Old gamma brightness is currently in use */
+#define SiS_SD3_MFBALLOWOFFCL  0x00000002   /* Supports off'ing CRTx in MFB if a clone mode is active */
+#define SiS_SD3_SUPPORTVBF34   0x00000004   /* Supports VBFlags3 and VBFlags4 */
+#define SiS_SD3_SUPPORTDUALDVI 0x00000008   /* Supports dual dvi-d (for future use) */
+#define SiS_SD3_SUPPORTDUALTV  0x00000010   /* Supports dual tv (for future use) */
+#define SiS_SD3_NEWOUTPUTSW    0x00000020   /* Supports NEWSETVBFLAGS (for future use) */
+#define SiS_SD3_CRT1SATGAIN    0x00000040   /* Supports CRT1 saturation gain */
+#define SiS_SD3_CRT2SATGAIN    0x00000080   /* Supports CRT2 saturation gain (apart from TV, see SiS_SD2_SUPPTVSAT) */
+
 #define SIS_DIRECTKEY          0x03145792
 
 /* SiSCtrl: Check mode for CRT2 */
@@ -643,6 +693,8 @@ typedef unsigned char  UChar;
 #define SiS_CF2_TVHIVISION   SiS_CF2_TVPAL
 #define SiS_CF2_TVYPBPR525I  SiS_CF2_TVNTSC
 #define SiS_CF2_TVYPBPR525P  (SiS_CF2_TVPAL | SiS_CF2_TVNTSC)
+#define SiS_CF2_TVYPBPR625I  SiS_CF2_TVPALN
+#define SiS_CF2_TVYPBPR625P  (SiS_CF2_TVPALN | SiS_CF2_TVPAL)
 #define SiS_CF2_TVYPBPR750P  SiS_CF2_TVPALM
 #define SiS_CF2_TVYPBPR1080I (SiS_CF2_TVPALM | SiS_CF2_TVPAL)
 
@@ -730,6 +782,12 @@ typedef struct {
 /* For extended memcpy() */
 typedef void (*vidCopyFunc)(UChar *, const UChar *, int);
 
+#ifdef SISISXORG6899900
+#define SISAGPHTYPE drm_handle_t
+#else
+#define SISAGPHTYPE ULong
+#endif
+
 /* Dual head private entity structure */
 #ifdef SISDUALHEAD
 typedef struct {
@@ -737,7 +795,8 @@ typedef struct {
     ScrnInfoPtr		pScrn_2;
     UChar		*BIOS;
     struct SiS_Private	*SiS_Pr;
-    ULong		agpHandle;
+#ifdef XF86DRI
+    SISAGPHTYPE		agpHandle;
     ULong		agpAddr;
     UChar		*agpBase;
     unsigned int	agpSize;
@@ -751,7 +810,6 @@ typedef struct {
     UChar       	*agpVtxBufBase;
     unsigned int	agpVtxBufSize;
     unsigned int	agpVtxBufFree;
-#ifdef XF86DRI
     sisRegion		agp;
     int			drmSubFD;
 #endif
@@ -816,6 +874,8 @@ typedef struct {
     int			sistvyfilter;
     int			tvxpos, tvypos;
     int			tvxscale, tvyscale;
+    int			siscrt1satgain;
+    Bool		crt1satgaingiven;
     int			ForceTVType, SenseYPbPr;
     unsigned int	ForceYPbPrType, ForceYPbPrAR;
     int			chtvtype;
@@ -831,7 +891,9 @@ typedef struct {
     Bool		restorebyset;
     Bool		CRT1gamma, CRT1gammaGiven, CRT2gamma, XvGamma, XvGammaGiven, XvDefAdaptorBlit;
     int			XvGammaRed, XvGammaGreen, XvGammaBlue;
-    int			GammaBriR, GammaBriG, GammaBriB;	/* strictly for Xinerama */
+    int			GammaBriR, GammaBriG, GammaBriB;		/* strictly for Xinerama */
+    float		NewGammaBriR, NewGammaBriG, NewGammaBriB;	/* strictly for Xinerama */
+    float		NewGammaConR, NewGammaConG, NewGammaConB;	/* strictly for Xinerama */
     unsigned int	CRT1MonGamma, CRT2MonGamma;
     unsigned int	CRT1VGAMonitorGamma, CRT2LCDMonitorGamma, CRT2VGAMonitorGamma;
     int			curxvcrtnum;
@@ -943,10 +1005,13 @@ typedef struct {
     UChar		oldCR32, oldCR36, oldCR37;
     UChar		myCR32, myCR36, myCR37, myCR63;
     UChar		newCR32;
-    unsigned int	VBFlags;		/* Video bridge configuration */
+    unsigned int	VBFlags;		/* Video bridge configuration (dynamic) */
     unsigned int	VBFlags2;		/* Video bridge configuration 2 (static flags only) */
+    unsigned int	VBFlags3, VBFlags4;	/* Video bridge configuration 3, 4 (dynamic) */
     unsigned int	VBFlags_backup;		/* Backup for SlaveMode-modes */
-    unsigned int	VBLCDFlags;		/* Moved LCD panel size bits here */
+    unsigned int	VBFlags_backup3;	/* Backup for SlaveMode-modes */
+    unsigned int	VBFlags_backup4;	/* Backup for SlaveMode-modes */
+    unsigned int	VBLCDFlags, VBLCDFlags2;
     int			ChrontelType;		/* CHRONTEL_700x or CHRONTEL_701x */
     unsigned int	PDC, PDCA;		/* PanelDelayCompensation */
     short		scrnOffset;		/* Screen pitch (data) */
@@ -980,9 +1045,9 @@ typedef struct {
 #endif
 #ifdef SIS_USE_EXA
     ExaDriverPtr	EXADriverPtr;
-    int			fillPitch, fillBpp, fillXoffs;
+    int			fillPitch, fillBpp;
     CARD32		fillDstBase;
-    int			copySXoffs, copyDXoffs, copyBpp;
+    int			copyBpp;
     int			copySPitch, copyDPitch;
     CARD32		copySrcBase, copyDstBase;
     int			copyXdir, copyYdir;
@@ -1010,7 +1075,8 @@ typedef struct {
     unsigned int	cmdQueueSize_div2;
     unsigned int	cmdQueueSize_div4;
     unsigned int	cmdQueueSize_4_3;
-    ULong 		agpHandle;
+#ifdef XF86DRI
+    SISAGPHTYPE		agpHandle;
     ULong		agpAddr;
     UChar 		*agpBase;
     unsigned int	agpSize;
@@ -1024,7 +1090,6 @@ typedef struct {
     UChar		*agpVtxBufBase;
     unsigned int	agpVtxBufSize;
     unsigned int	agpVtxBufFree;
-#ifdef XF86DRI
     sisRegion		agp;
 #endif
     Bool		AGPInitOK;
@@ -1163,6 +1228,8 @@ typedef struct {
     Bool		sisfb_haveemi, sisfb_haveemilcd, sisfb_tvposvalid, sisfb_havelock;
     UChar		sisfb_emi30,sisfb_emi31,sisfb_emi32,sisfb_emi33;
     int			sisfb_tvxpos, sisfb_tvypos;
+    int			siscrt1satgain;
+    Bool		crt1satgaingiven;
     Bool		sisfbHaveNewHeapDef;
     unsigned int	sisfbHeapSize, sisfbVideoOffset;
     Bool		sisfbxSTN;
@@ -1222,7 +1289,7 @@ typedef struct {
     SIS_CP_H
 #endif
     ULong		ChipFlags;
-    ULong		SiS_SD_Flags, SiS_SD2_Flags;
+    ULong		SiS_SD_Flags, SiS_SD2_Flags, SiS_SD3_Flags, SiS_SD4_Flags;
     Bool		UseHWARGBCursor;
     int			OptUseColorCursor;
     int			OptUseColorCursorBlend;
@@ -1295,6 +1362,10 @@ typedef struct {
     float		GammaR2, GammaG2, GammaB2;
     int			GammaR2i, GammaG2i, GammaB2i;
     int			GammaBriR2, GammaBriG2, GammaBriB2;
+    float		NewGammaBriR, NewGammaBriG, NewGammaBriB;
+    float		NewGammaConR, NewGammaConG, NewGammaConB;
+    float		NewGammaBriR2, NewGammaBriG2, NewGammaBriB2;
+    float		NewGammaConR2, NewGammaConG2, NewGammaConB2;
     ExtensionEntry	*SiSCtrlExtEntry;
     char		devsectname[32];
     Bool		SCLogQuiet;
@@ -1491,6 +1562,8 @@ extern int   SiS_GetTVxposoffset(ScrnInfoPtr pScrn);
 extern int   SiS_GetTVyposoffset(ScrnInfoPtr pScrn);
 extern int   SiS_GetTVxscale(ScrnInfoPtr pScrn);
 extern int   SiS_GetTVyscale(ScrnInfoPtr pScrn);
+extern int   SiS_GetSISCRT1SaturationGain(ScrnInfoPtr pScrn);
+extern void  SiS_SetSISCRT1SaturationGain(ScrnInfoPtr pScrn, int val);
 
 #endif  /* _SIS_H_ */
 
